import pygame, random
import serial
from pylab import *
from time import sleep
from threading import Thread

def MakeVector(a, b):
    a = mat(a)
    b = mat(b)
    return mat(b-a).T

def Acos(x):
    return arccos(x)*180/pi

def FindAngle(a, b):
    return Acos(a.T*b/norm(a)/norm(b))[0,0]

def FindDirection(a, b):
    a = concatenate((a, mat([0])), axis=0)
    b = concatenate((b, mat([0])), axis=0)
    if cross(a.T, b.T)[0, 2] < 0:
        return "L"
    else:
        return "R"

# reducing the number of points
def DouglasPeucker(ps, epsilon):

    def DistanceToSegment(u, v):
        return norm( u - ( (v/norm(v)).T * u)[0,0] * (v/norm(v)) )
    
    dmax = 0
    index = -1
    end = len(ps)
    for i in range(1, end):
        d = DistanceToSegment(MakeVector(ps[0], ps[i]), MakeVector(ps[0], ps[end-1]))
        if d > dmax:
            index = i
            dmax = d
    
    if dmax > epsilon:
        rec_result_1 = DouglasPeucker(ps[:index+1], epsilon)
        rec_result_2 = DouglasPeucker(ps[index:], epsilon)

        rs = rec_result_1[:-1] + rec_result_2

    else:
        rs = [ps[0], ps[end-1]]

    return rs


def RoundLine(srf, color, start, end, radius=1):
    dx = end[0]-start[0]
    dy = end[1]-start[1]
    distance = max(abs(dx), abs(dy))
    for i in range(distance):
        x = int( start[0]+float(i)/distance*dx)
        y = int( start[1]+float(i)/distance*dy)
        pygame.draw.circle(srf, color, (x, y), radius)

class Display(Thread):
    def __init__(self, path, simple_path = None):
        Thread.__init__(self, name = "Display Thread")
        self._path = path
        self._simple_path = simple_path

    def Showing(self, path):
        xx = [x for x, y in path]
        yy = [y for x, y in path]
        plot(xx, yy, 'o')

    def run(self):
        
        subplot(2, 1, 1)
        xlim([-10, W+10])
        ylim([-10, H+10])
        self.Showing(self._path)
        
        if self._simple_path != None:
            subplot(2, 1, 2)
            xlim([-10, W+10])
            ylim([-10, H+10])
            self.Showing(self._simple_path)
        
        show()
    

def Run(ps):
    T = 0.5
    s = serial.Serial('/dev/ttyUSB0', 9600)
    s.open()
    a = MakeVector(ps[0], ps[1])

    dis = D*norm(a)
    print len(ps)
    for x in range(1, len(ps)-1): 
        print "############# %3d ################" %(x)
        print "%f => F%03d" %(norm(a), dis)
        if dis < 150:
            s.write("F%03d" %(dis))
            print s.readline().strip()
            print s.readline().strip()
            print s.readline().strip()
        else:
            while dis > 150:
                s.write("F%03d" %(150))
                print s.readline().strip()
                print s.readline().strip()
                print s.readline().strip()
                dis -= 150
            s.write("F%03d" %(dis))
            print s.readline().strip()
            print s.readline().strip()
            print s.readline().strip()
       
        b = MakeVector(ps[x], ps[x+1]);
        angle = FindAngle(a, b)
        angle = angle/90.0 * 50

        direction = FindDirection(a, b)
        print "%f => %s%03d" %(angle, direction, int(angle))
        s.write("%s%03d" %(direction, int(angle)))
        print s.readline().strip()
        print s.readline().strip()
        print s.readline().strip()

        a = b
        dis = D*norm(a)
    
    print "############# %3d ################" %(len(ps)-1)
    print "%f => F%03d" %(norm(a), dis)
    s.write("F%03d" %(dis))
    print s.readline().strip()
    print s.readline().strip()
    print s.readline().strip()
    print "##################################"

    s.close()

def Drawing():
    
    path = []
    draw_on = False
    last_pos = (0, 0)
    color = (0, 128, 255)
    radius = 5

    screen = pygame.display.set_mode((W,H))
    try:
        while True:
            e = pygame.event.wait()
            if e.type == pygame.QUIT:
                raise StopIteration
        
            if e.type == pygame.KEYDOWN:

                # press 'space' to start car
                if e.key == pygame.K_SPACE:
                    if len(path) > 1:
                        simple_path = DouglasPeucker(path, 2)
                        display = Display(path, simple_path)
                        display.start()
                        Run(simple_path)
                        screen.fill([0, 0, 0])
                        path = []
                
                # press 'c' to clear screen
                elif e.key == pygame.K_c:
                    screen.fill([0, 0, 0])
                    path = []
                    
                # press 'm' to change mode
                elif e.key == pygame.K_m:
                    pass
                                        

            if e.type == pygame.MOUSEBUTTONDOWN:
                pygame.draw.circle(screen, color, e.pos, radius)
                pos = (e.pos[0], H-e.pos[1])
                path.append(pos)
                draw_on = True
            if e.type == pygame.MOUSEBUTTONUP:
                draw_on = False
            if e.type == pygame.MOUSEMOTION:
                if draw_on:
                    pygame.draw.circle(screen, color, e.pos, radius)
                    pos = (e.pos[0], H-e.pos[1])
                    path.append(pos)
                    RoundLine(screen, color, e.pos, last_pos,  radius)
                last_pos = e.pos
            pygame.display.flip()

    except StopIteration:
        pass

    pygame.quit()

def main():
    global W, H, T
    W = 800
    H = 600
    T = 3
    
    global D, A
    D = 0.2
    A = 0.57

    Drawing()
    
     
if __name__ == '__main__':
    main()

