#include <stdio.h>
#include <motelib/system.h>
#include <motelib/led.h>
#include <motelib/uart.h>
#include <AppHardwareApi.h>
#include <motelib/timer.h>
#include <motelib/radio.h>

#define TIME_TO_STOP 100000000

#define START_SPEED         40

#define DELAY_UNIT      100000

char data[5];
uint32_t start_time;
uint8_t speed;
Timer t;

void receiveCmd(Address src, MessageType type, void *msg, uint8_t len);
void initTickTimer();
void stopWheel(uint32_t u32DeviceId, uint32_t u32ItemBitmap);
void wheelRotate(uint8_t l, uint8_t r);
uint8_t stringToInt(char *s, uint8_t len);

void boot() {

  start_time = 0;
  speed = START_SPEED;
  uartEnable(true, true);

  printf("RUNNING...\n\r");

  // TickTimer
  vAHI_TickTimerConfigure(E_AHI_TICK_TIMER_DISABLE);
  vAHI_TickTimerIntEnable(TRUE);
  vAHI_TickTimerRegisterCallback(stopWheel);
  vAHI_TickTimerWrite(TIME_TO_STOP - 1000);
  vAHI_TickTimerInterval(TIME_TO_STOP);
  vAHI_TickTimerConfigure(E_AHI_TICK_TIMER_STOP);

  // PWM for speed control
  vAHI_TimerEnable(E_AHI_TIMER_4, // identity of timer
  		   10,            // prescale, range from 0 to 16
  		   TRUE,          // low-to-high interrupt
  		   TRUE,          // high-to-low interrupt
  		   TRUE);         // output enable
  vAHI_TimerConfigureOutputs(E_AHI_TIMER_4, // identity of timer
  			     FALSE,         // PWM ouput inversion
  			     FALSE);        // disable external gating input
  vAHI_TimerStartRepeat(E_AHI_TIMER_4, // identity of timer
  			1,             // time to rise(low-to-high)
  			1);            // time of fall(high-to-low, time from the begining of the signal)
  
  vAHI_TimerEnable(E_AHI_TIMER_3, 10, TRUE, TRUE, TRUE);
  vAHI_TimerConfigureOutputs(E_AHI_TIMER_3, FALSE, FALSE);
  vAHI_TimerStartRepeat(E_AHI_TIMER_3, 1, 1);

  // radio handler
  radioSetRxHandler(receiveCmd);
}

void receiveCmd(Address src, MessageType type, void *msg, uint8_t len) {
  uint32_t distance;
  uint32_t wait_time;
  ledSetValue(7);
  data[0] = ((char*) msg)[0];
  data[1] = ((char*) msg)[1];
  data[2] = ((char*) msg)[2];
  data[3] = ((char*) msg)[3];
  data[4] = '\0';

  distance = stringToInt(data+1, 3);
  
  wait_time = distance*100/speed;
  wait_time *= DELAY_UNIT;

  start_time = TIME_TO_STOP - wait_time;

  if(data[0] == 'L'){  
    printf("####### L ########\n\r");
    speed = START_SPEED;
    wheelRotate(1, 0);
    initTickTimer();
  }
  else if(data[0] == 'R') {
    printf("####### R ########\n\r");
    speed = START_SPEED;
    wheelRotate(0, 1);
    initTickTimer();
  }
  else if(data[0] == 'F') {
    printf("####### F ########\n\r");
    speed = START_SPEED;
    wheelRotate(1, 1);
    initTickTimer();
  }    
  printf("distance %8d\n\r", distance);

}

void initTickTimer() {
  printf("start_time %d\n\r", start_time);
  vAHI_TickTimerWrite(start_time);
  vAHI_TickTimerIntEnable(TRUE);
}

void stopWheel(uint32_t u32DeviceId, uint32_t u32ItemBitmap) {
  ledSetValue(0);
  wheelRotate(0, 0);
  printf("##################\n\r");
  radioRequestTx(1, 21, &start_time, sizeof(start_time), NULL);
}

void wheelRotate(uint8_t l, uint8_t r) {
  printf("speed %d: %d %d\n\r", speed, l, r);  
  if(l != 0) vAHI_TimerStartRepeat(E_AHI_TIMER_4, 100-speed, 100);
  else       vAHI_TimerStartRepeat(E_AHI_TIMER_4, 100, 100);

  if(r != 0) vAHI_TimerStartRepeat(E_AHI_TIMER_3, 100-speed, 100);
  else       vAHI_TimerStartRepeat(E_AHI_TIMER_3, 100, 100);
}

uint8_t stringToInt(char *s, uint8_t len) {
  uint16_t value = 0;
  uint8_t i;
  for(i = 0; i < len; ++i) {
    value *= 10;
    s[i] = s[i] - '0';
    value += s[i];
  }
  return value;
}
